const {findAvailablePort} = require("../utilities/findAvailablePort");
const {input} = require("@inquirer/prompts");
const querystring = require("querystring");
const {nanoid} = require("nanoid");
const express = require('express')
const {setAuth, hasAuth, authFile} = require("../utilities/authentication");

exports.command = 'authenticate'
exports.description = 'starts the Spotity authentication process'
exports.builder = function (argv) {
    return argv.option('client-id')
}
;
exports.handler = async function (argv) {
    if (hasAuth()) {
        console.log(`spotify.local is already authenticated, remove "${authFile}" is you want to log out.`);
        return;
    }

    const port = await findAvailablePort();

    const authenticateRoute = 'authenticate';
    const authenticateUrl = `http://localhost:${port}/${authenticateRoute}`;

    const tutorial = [
        'Visit https://developer.spotify.com/dashboard and click "Create app"',
        'Set the "App name" as "spotify.local"',
        'Set the "App description" as "My spotify.local"',
        'Set the "Website" as "https://code.tmp.dev/ian/spotify.local"',
        `Set the "Redirect URI" as "${authenticateUrl}"`,
        'Select under "Which API/SDKs are you planning to use?" the "Web API"',
        'Sign your life away by agreeing with the "Developer Terms of Service and Design Guidelines"',
        'Click on the "Save" button',
        'Copy the "Client ID" from the settings page',
    ];

    tutorial.forEach((string, index) => console.log(`${index+1} ${string}`));

    const clientId = argv.clientId ?? await input({ message: `${tutorial.length}. Paste the clientId here` })
    const originalState = nanoid();
    const redirectUrl = new URL('https://accounts.spotify.com/authorize');
    redirectUrl.search = querystring.stringify({
        response_type: 'code',
        client_id: clientId,
        scope: [
            'user-library-read',
            'user-read-private',
            'user-library-modify',
            'user-follow-modify',
            'user-follow-read',
            'playlist-read-private',
            'playlist-modify-public',
            'playlist-modify-private',
            'playlist-read-collaborative',
            'user-top-read',
            'user-read-recently-played',
            'user-read-playback-state',
            'user-modify-playback-state',
            'user-read-currently-playing',
            'user-read-playback-position'
        ].join(' '),
        redirect_uri: authenticateUrl,
        state:  originalState
    });

    const app = express();

    let server;
    app.get(`/${authenticateRoute}`, (req, res) => {
        const {state, code} = req.query;
        if (originalState === state) {
            setAuth(code);

            res.send('<script>(() => close())()</script>');

            server?.close();

            console.log(`${tutorial.length+2}. Spotify.local has been authenticated`);
        } else {
            res.status(400);
            res.send('NO')
        }
    });

    server = app.listen(port, () => console.log(`${tutorial.length+1}. Follow the following URL to authenticate: ${redirectUrl}`));
}